import math
import numpy as np
import matplotlib.pyplot as plt
import collections
import datetime as dt
from matplotlib import cm
from scipy.spatial import Voronoi, voronoi_plot_2d

from math import radians, cos, sin, asin, sqrt, atan2, degrees, atan
cm2m = 0.01 #converting cm to m
sec2min = 60 #converting sec to min
measIntervalInMinutes = 5

def frequency(array): #for testing
    values, array = np.unique(array, return_inverse=True)
    return values, np.bincount(array.ravel())

#apply meltrate from temp formula
def meltrate(vol, temp):
    rosw = 1027
    oc_heatCap = 4000
    ice_heatCap = 2090
    freezeTemp = 2.498
    basalTemp = -15
    latentHeat = 334000 
    theta_out = -2.09
    m_rate = vol*rosw*temp*oc_heatCap/(latentHeat+oc_heatCap*(theta_out-freezeTemp)+ice_heatCap*(freezeTemp-basalTemp))/2
    return m_rate

#apply meltrate from salinity formula
def meltrateSalinity_new(vol, salIn, salOut):
    rosw = 1027
    fwExchange = vol*rosw*((salIn - salOut)/ salOut)
    return fwExchange

#apply meltrate from salinity formula
def meltrateSalinity(vol, sal):
    rosw = 1027
    #salIn=34.53 
    salOut=34.45
    fwExchange = vol*rosw*((sal - salOut)/ salOut)
    return fwExchange
    
def count_good_bad_values():
    from sklearn.impute import MissingIndicator
    indicator = MissingIndicator(missing_values=-999)
    result=indicator.fit_transform(feature_array)
    print frequency(result) #shows #s of good/bad data    
    
def test_all(j,volume,Di): # sum of volumes for all data
   tot_vol=0
   tot_abs_vol=0
   tot_ang=0
   tot_abs_ang=0
   for i in range(j-1):
     #if file_name[i]=='rcm10869.dat':
       tot_vol += volume[i]
       tot_abs_vol += abs(volume[i])
       tot_ang += sin(Di[i])
       tot_abs_ang+= abs(sin(Di[i]))

   #print  tot_vol, tot_abs_vol, tot_vol/tot_abs_vol#, tot_abs_ang/j 
   print "total flux: ", tot_abs_vol/10**12, ".10^12"

#for f in file_list:
#    print f, point_index[f],X_coord[point_index[f]], Y_coord[point_index[f]], area[point_index[f]]

def test_hour(s): #check sum of volumes for a given hour
    l=feature_dict[s]
    lsum=0
    llsum=0
    for x in l:
        lsum+=x
        llsum+=abs(x)
    print lsum/llsum
 
    
def polygon_area(vertices):
    r"""Shoelace formula for computing area of polygon
    .. math::
        A = \sum_{i=1}^{n} x_i \left(y_{i+1} - y_{i-1}\right), \quad i\mod n
    References:
        - https://en.wikipedia.org/wiki/Shoelace_formula
        - https://stackoverflow.com/questions/451426/how-do-i-calculate-the-area-of-a-2d-polygon/717367#717367
        - https://stackoverflow.com/questions/24467972/calculate-area-of-polygon-given-x-y-coordinates
    Args:
        vertices (numpy.ndarray): Vertices of the polygon
    Returns:
        float: Area of the polygon
    """
    n = len(vertices)
    area = 0.0
    if n < 3:
        return area
    x, y = vertices[:, 0], vertices[:, 1]
    for i in range(1, n-1):
        area += x[i] * (y[i + 1] - y[i - 1])
    # i=n-1
    area += x[n-1] * (y[0] - y[n - 2])
    # i=n
    area += x[0] * (y[1] - y[n - 1])
    return 0.5 * abs(area)


def haversine(lon1, lat1, lon2, lat2):
    """
    Calculate the great circle distance between two points 
    on the earth (specified in decimal degrees)
    """
    # convert decimal degrees to radians 
    lon1, lat1, lon2, lat2 = map(radians, [lon1, lat1, lon2, lat2])
    # haversine formula 
    dlon = lon2 - lon1 
    dlat = lat2 - lat1 
    a = sin(dlat/2)**2 + cos(lat1) * cos(lat2) * sin(dlon/2)**2
    c = 2 * asin(sqrt(a))
    # Radius of earth in kilometers is 6371
    km = 6371* c
    return km
    
def calculate_bearing(lat1,lon1,lat2,lon2):
    """
    Calculates the bearing between two points.
    """

    lat1 = math.radians(lat1)
    lat2 = math.radians(lat2)

    diffLong = math.radians(lon2 - lon1)

    x = math.sin(diffLong) * math.cos(lat2)
    y = math.cos(lat1) * math.sin(lat2) - (math.sin(lat1)
            * math.cos(lat2) * math.cos(diffLong))

    initial_bearing = math.atan2(x, y)

    initial_bearing = math.degrees(initial_bearing)
    bearing = (initial_bearing + 360) % 360

    return bearing

def nums(line):
    lst = []
    for l in line.split():
              try:
                  lst.append(float(l))
              except ValueError:
                      pass
    return lst 

def myImputer(feature_array):
    prev_valid_val = 0
    counter = 0
    for col in range(feature_array.shape[1]):
        for row in range(feature_array.shape[0]):
            if feature_array[row][col] == -999:
                counter+=1
                feature_array[row][col] = prev_valid_val
            else:
                prev_valid_val = feature_array[row][col]
    print "number of invalid values fixed: ", counter
    return feature_array

def myDictImputer(feature_dict):
    dates = feature_dict.keys()
    dates.sort()
    instr = feature_dict[dates[1000]].keys()
    prev_valid_val = 0
    print instr
    counter = 0
    print dates[0]
    for col in instr:
        for row in dates:
            if (col not in feature_dict[row]) or (feature_dict[row][col] == -999):
                counter+=1
                feature_dict[row][col] = prev_valid_val
            else:
                prev_valid_val = feature_dict[row][col]
    print "number of invalid values fixed: ", counter
    return feature_dict


def plotVorVolumes(vor,regions,feature_dict):
    volume = np.zeros(len(regions))
    counter=np.zeros(len(regions))
    for key, value in feature_dict.iteritems():
        for i in range(len(value)):
           if value[i] != -999:
            volume[i] += value[i]
            counter[i] += 1
    volume = volume / counter
   
    # find min/max values for normalization
    minima = min(volume)
    maxima = -0.85*minima#max(volume)
    print "minima:",minima," maxima:",maxima
    norm = cm.colors.SymLogNorm(linthresh=3*10**9, linscale=0.1, vmin=minima, vmax=maxima)
    
    mapper = cm.ScalarMappable(norm=norm, cmap=cm.coolwarm)
    voronoi_plot_2d(vor,show_vertices=False, s=1)
    for r in range(len(regions)):
        region = vor.regions[vor.point_region[r]]
        if not -1 in region:
            polygon = [vor.vertices[i] for i in region]
            plt.fill(*zip(*polygon), color=mapper.to_rgba(volume[r]))
    mapper.set_array([])
    cbar = plt.colorbar(mapper)
    m1='-1.10^8';   m2='1.10^8'                # colorbar midpoint value
    cbar.set_ticks([minima,maxima])
    cbar.set_ticklabels([m1,m2])
    plt.xlabel("Distance (m)")
    plt.ylabel("Depth (m)")
    plt.show()

def plotVorVolumesSmooth(vor,regions,feature_dict,neighb,point_index):
    volume = np.zeros(len(regions))
    counter=np.zeros(len(regions))
    for key, value in feature_dict.iteritems():
        for i in range(len(value)):
           if value[i] != -999:
            volume[i] += value[i]
            counter[i] += 1
    volume = volume / counter
   
    # find min/max values for normalization
    minima = min(volume)
    maxima = -0.85*minima#max(volume)
    minima=0.88*minima
    maxima=0.7*maxima
    print "minima:",minima," maxima:",maxima
    norm = cm.colors.SymLogNorm(linthresh=3*10**9, linscale=0.1, vmin=minima, vmax=maxima)
    
    mapper = cm.ScalarMappable(norm=norm, cmap=cm.coolwarm)
    voronoi_plot_2d(vor,show_vertices=False, s=1)
    for r in range(len(regions)):
        region = vor.regions[vor.point_region[r]]
        if not -1 in region:
            for i in range(len(region)):
                #v0 = vor.region[r]
                v1 = region[i] 
                if i<len(region)-1: v2 = region[i+1] 
                else: v2 = region[0] 
                val0, val1, val2 = estimate_value(volume, r, neighb[v1], neighb[v2],point_index)
                T=(val0+val1+val2)/3
                #print T,
                polygon = [vor.points[r]] + [vor.vertices[i] for i in [v1,v2]]
                plt.fill(*zip(*polygon), color=mapper.to_rgba(T))	
    mapper.set_array([])
    cbar = plt.colorbar(mapper)
    m1='-1.10^8';   m2='1.10^8'                # colorbar midpoint value
    cbar.set_ticks([minima,maxima])
    cbar.set_ticklabels([m1,m2])
    plt.xlabel("Distance (m)")
    plt.ylabel("Depth (m)")
    plt.show()

def plotVorTemps(vor,regions,feature_dict):
    temp = np.zeros(len(regions))
    counter=np.zeros(len(regions))
    mn =99999;mx=-99999
    #mn = -2.04; mx = -1.76
    mn = -1.98; mx = -1.81
    for key, value in feature_dict.iteritems():
        for i,key in enumerate(value):#range(len(value)): 
          if value[key] > -990:
            temp[i] += value[key]
            counter[i] += 1
            #if value[key]<mn: mn=value[key]
            #if value[key]>mx: mx=value[key]
    for i in range(len(regions)):
        if counter[i] > 0:
            temp[i] /= counter[i]
            #if temp[i]<mn: mn = temp[i]
            #if temp[i]>mx: mx = temp[i]
        else: temp[i] = (mn+mx)/2
    print 'temp;', temp, temp.min(), temp.max()
    # use fixed mn and mx for consistency among plots
    norm = cm.colors.Normalize(vmin=mn, vmax=mx)
    
    mapper = cm.ScalarMappable(norm=norm, cmap=cm.coolwarm)
    voronoi_plot_2d(vor,show_vertices=False, s=1)
    for r in range(len(regions)):
        region = vor.regions[vor.point_region[r]]
        if not -1 in region:
            polygon = [vor.vertices[i] for i in region]
            plt.fill(*zip(*polygon), color=mapper.to_rgba(temp[r]))
    mapper.set_array([])
    cbar = plt.colorbar(mapper)
    #m1=mn;   m2=mx                # colorbar midpoint value
    #m1='-2.2';   m2='-1.27'                # colorbar midpoint value
    cbar.set_ticks([mn,mx])
    cbar.set_ticklabels([str(round(mn,2)),str(round(mx,2))])
    plt.xlabel("Distance (m)")
    plt.ylabel("Depth (m)")
    plt.show()

def  estimate_value(temp, v, nb1, nb2,ind):
    val0 = temp[v]
    val1 = 0
    count = 0
    for w in nb1:
        if temp[ind[w]] > -0.01: temp[ind[w]] = -1.76
        val1 += temp[ind[w]]
        count += 1
    if count > 0: val1 /= count
    val2 = 0
    count = 0
    for w in nb2:
        if temp[ind[w]] > -0.01: temp[ind[w]] = -1.76 
        val2 += temp[ind[w]]
        count += 1
    if count > 0: val2 /= count
    return val0, val1, val2 # values at thee triangle points

def plotVorTemps2(vor,regions,feature_dict,neighb,point_index):
    temp = np.zeros(len(regions))
    counter=np.zeros(len(regions))
    mn =99999;mx=-99999
    mn = -1.98; mx = -1.81
    for key, value in feature_dict.iteritems():
        for i,key in enumerate(value):#range(len(value)): 
          if value[key] > -990:
            temp[i] += value[key]
            counter[i] += 1
            #if value[key]<mn: mn=value[key]
            #if value[key]>mx: mx=value[key]
    for i in range(len(regions)):
        if counter[i] > 0:
            temp[i] /= counter[i]
            #if temp[i]<mn: mn = temp[i]
            #if temp[i]>mx: mx = temp[i]
        else: temp[i] = (mn+mx)/2

    print 'temp;', temp
    # use fixed mn and mx for consistency among plots
    norm = cm.colors.Normalize(vmin=mn, vmax=mx)
    
    mapper = cm.ScalarMappable(norm=norm, cmap=cm.coolwarm)
    voronoi_plot_2d(vor,show_vertices=False, s=1)
    #for i in range(len(vor.vertices)):
        #plt.text(vor.vertices[i][0],vor.vertices[i][1],i)
    #for i in range(len(vor.points)):
        #plt.text(vor.points[i][0],vor.points[i][1],i)
    for r in range(len(regions)):
        region = vor.regions[vor.point_region[r]]
        if not -1 in region:
            for i in range(len(region)):
                #v0 = vor.region[r]
                v1 = region[i] 
                if i<len(region)-1: v2 = region[i+1] 
                else: v2 = region[0] 
                val0, val1, val2 = estimate_value(temp, r, neighb[v1], neighb[v2],point_index)
                T=(val0+val1+val2)/3
                #print T,
                polygon = [vor.points[r]] + [vor.vertices[i] for i in [v1,v2]]
                plt.fill(*zip(*polygon), color=mapper.to_rgba(T))	

    mapper.set_array([])
    cbar = plt.colorbar(mapper)
    #m1=mn;   m2=mx                # colorbar midpoint value
    #m1='-2.2';   m2='-1.27'                # colorbar midpoint value
    cbar.set_ticks([mn,mx])
    cbar.set_ticklabels([str(round(mn,2)),str(round(mx,2))])
    plt.xlabel("Distance (m)")
    plt.ylabel("Depth (m)")
    plt.show()

def plotDict(Dict):
    Dict = collections.OrderedDict(sorted(Dict.items()))
    x = []
    y = []
    for time_ind in Dict:
        x += [time_ind]
        y += [Dict[time_ind]]

    #compute moving average
    window = 60
    win2 = window/2
    yy=np.array(y)
    print "test: ", sum(y)*60/10**12
    for i in range(win2):
        ysum = 0
        for j in range(i-win2,i+win2): 
            if j>=0: ysum += yy[j]
            else: ysum += yy[len(y)+j]
        y[i] = ysum/window
    
    for i in range(win2, len(y)-win2):#len(y)):
        ysum = 0
        for j in range(i-win2,i+win2): 
            ysum += yy[j]
        y[i] = ysum/window

    for i in range(len(y)-win2, len(y)):#len(y)):
        ysum = 0
        for j in range(i-win2,i+win2): 
            if j<len(y):ysum += yy[j]
            else: ysum += yy[j-len(y)]
        y[i] = ysum/window

    print "test: ", sum(y)*60/10**12
    plt.plot(x, y)
    plt.ylabel('Temp')
    plt.xlabel('Time')
    plt.grid()
    plt.show 

def plotMelt(meltDict, label):
    zoom = 5
    numInstr = 21
    meltDict = collections.OrderedDict(sorted(meltDict.items()))
    x = []
    y = []
    for time_ind in meltDict:
        x += [time_ind]
        y += [measIntervalInMinutes*sec2min*cm2m*meltDict[time_ind]*zoom] #numInstr*

    #compute moving average
    window = 60
    win2 = window/2
    yy=np.array(y)
    print "test: ", sum(y)*60/10**12
    for i in range(win2):
        ysum = 0
        for j in range(i-win2,i+win2): 
            if j>=0: ysum += yy[j]
            else: ysum += yy[len(y)+j]
        y[i] = ysum/window
    
    for i in range(win2, len(y)-win2):#len(y)):
        ysum = 0
        for j in range(i-win2,i+win2): 
            ysum += yy[j]
        y[i] = ysum/window

    for i in range(len(y)-win2, len(y)):#len(y)):
        ysum = 0
        for j in range(i-win2,i+win2): 
            if j<len(y):ysum += yy[j]
            else: ysum += yy[j-len(y)]
        y[i] = ysum/window

    z = []
    s = 0
    for l in y:
        s += l/zoom
        z += [s]

    print "test: ", sum(y)*60/10**12, " total melt:", round(s/10**12,2), "*10^12"
    plt.plot(x, y)
    plt.plot(x, z)
    plt.ylabel(label)
    plt.xlabel('Time')
    plt.grid()
    plt.show
    
    
